/******************************************************************NVMH3****
File: $Id: //sw/devtools/ShaderLibrary/main/HLSL/EdgeFuzz.fx#1 $

Copyright NVIDIA Corporation 2007
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY
LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF
NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

% Simple edge lighting effect -- the color of the surface shifts
% when viewed on-edge. Optionally, the light color can also
% be effected -- the result of both is to create an illusion
% of the surface being covered by some sort of smooth 
% barely-visible fuzz.

keywords: material organic

******************************************************************************/

//float Script = 0.8;

//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 matWorldInv;
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;

/*********** Tweakables **********************/

/// Point Lamp 0 ////////////
float3 Lamp0Pos = {-500.0f,-500.0f,500.0f};
float3 Lamp0Color = {0.8f,0.8f,0.8f};
float3 AmbiColor = {0.2f, 0.2f, 0.2f};
/////
float3 SurfColor = {1.0f, 0.0f, 0.0f};
float3 EdgeColor = {0.0f, 0.0f, 1.0f};
float Ks = 0.9;
float SpecExpon = 100.0;
float sp = 0.6;
float edgeFade = 0.9;
float edgeFadeL = -.0;


/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldTangent	: TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	: TEXCOORD5;
};

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,matWorldInv).xyz;
    OUT.WorldTangent = mul(IN.Tangent,matWorldInv).xyz;
    OUT.WorldBinormal = mul(IN.Binormal,matWorldInv).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,matWorld).xyz;
    OUT.LightVec = (Lamp0Pos - Pw);
#ifdef FLIP_TEXTURE_Y
    OUT.UV = float2(IN.UV.x,(1.0-IN.UV.y));
#else /* !FLIP_TEXTURE_Y */
    OUT.UV = IN.UV.xy;
#endif /* !FLIP_TEXTURE_Y */
    OUT.WorldView = normalize(matViewInv[3].xyz - Pw);
    OUT.HPosition = mul(Po,matWorldViewProj);
    return OUT;
}

/********* pixel shader ********/

float4 edgePS(vertexOutput IN):COLOR {
    float3 Ln = normalize(IN.LightVec.xyz);
    float3 Vn = normalize(IN.WorldView);
    float3 Nn = normalize(IN.WorldNormal);
    float3 Hn = normalize(Vn + Ln);
    float ldn = dot(Ln,Nn);
    float4 lv = lit(ldn,dot(Hn,Nn),SpecExpon);
    float subd = abs(dot(Nn,Vn));
    subd = pow(subd,sp);
    float3 SurfaceColor = lerp(EdgeColor,SurfColor,subd);
    SurfaceColor = lerp(SurfColor,SurfaceColor,edgeFade);
    ldn = max(ldn,0);
    ldn = pow(ldn,sp);
    float3 IncidentColor = lerp(EdgeColor,Lamp0Color,ldn);
    IncidentColor = lerp(Lamp0Color,IncidentColor,edgeFadeL);
    float3 diffContrib = SurfaceColor * (lv.y*IncidentColor + AmbiColor);
    float3 specContrib = Ks * lv.y * lv.z * IncidentColor;
    //return SurfaceColor;
    return float4((diffContrib+specContrib).rgb,1);
}

/*************/

technique Main
{
    pass p0
	{		
	VertexShader = compile vs_2_0 std_VS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		//CullMode = None;
	PixelShader = compile ps_2_a edgePS();
	}
}

/***************************** eof ***/
