#include <transform>
#include <fog>
#include <pos>
#include <normal>
#include <tangent>
#define INDOOR
#ifdef INDOOR
	#include <light>
#else
	float4 vecSunDir;
	float4 vecSunColor;
#endif
float4 vecLight;

texture entSkin1;	// texture
texture entSkin2;	// normal map

const float facAmbient = 0.5; // factors to play with
const float facBump = 1.5;

sampler sBaseTex = sampler_state { Texture = <entSkin1>; };
sampler sBump = sampler_state { Texture = <entSkin2>; };

struct bumpOut
{
	float4 Pos:     POSITION;
	float  Fog:		FOG;
	float4 Color:	COLOR;
	float4 Ambient: COLOR1;
	float2 Tex:   	TEXCOORD0;
	float2 Bump:   TEXCOORD1;
	float3 Normal:	TEXCOORD2;
	float3 Light:	TEXCOORD3;
};

bumpOut bumpVS(
in float4 inPos: 	POSITION, 
in float3 inNormal:	NORMAL,
in float2 inTex: 	TEXCOORD0,
in float3 inTangent: TEXCOORD2)
{
	bumpOut Out;

	Out.Pos	= DoTransform(inPos);
	Out.Tex	= inTex;
	Out.Bump = inTex;	// different coordinates required for ps_1_1
	Out.Fog	= DoFog(inPos);
	Out.Ambient = facAmbient*vecLight;	
	
	float3 P = DoPos(inPos);
		
	CreateTangents(inNormal,inTangent);
	float3 N = matTangent[2];
		
	float3 lightVec;
	// transform the output values into the 0..1 range
	#ifdef INDOOR
		lightVec = normalize(vecLightPos[0].xyz - P);
		Out.Color = vecLightColor[0];
	#else
		lightVec = -vecSunDir;
		Out.Color = vecSunColor;
	#endif
	Out.Light = facBump*DoTangent(lightVec) * 0.5 + 0.5;
	Out.Normal = DoTangent(N) * 0.5 + 0.5;

	return Out;		
}


float4 bumpPS(bumpOut In): COLOR
{
	float4 base = tex2D(sBaseTex,In.Tex);
	float3 bumpNormal = tex2D(sBump,In.Bump);
	float diffuse = saturate(dot(In.Light*2 - 1,bumpNormal*2 - 1));
#define SHADOW
#ifdef SHADOW		
		diffuse *= saturate(4 * dot(In.Light*2 - 1,In.Normal*2 - 1));
#endif		
	return base * (diffuse * In.Color + In.Ambient);
}

technique bump
{
	pass one
	{		
		VertexShader = compile vs_1_1 bumpVS();
		PixelShader = compile ps_1_1 bumpPS();
	}
}

technique fallback { pass one { } }
