/*********************************************************************NVMH3****
File: $Id: //sw/devtools/ShaderLibrary/main/HLSL/checker3d_math.fx#1 $

Copyright NVIDIA Corporation 2007
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY
LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF
NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

% 3D Checker showing anti-aliasing using ddx/ddy.
% This result is PURELY numeric, so slower than using texture-based AA.
% It is, however, able to anti-alias regardless of the view scale.
% For a fast texture-based version, see "checker3d.fx"

keywords: material pattern

$Date: 2007/06/15 $

******************************************************************************/

//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 matWorldInv;
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;

/************************************************************/

float3 Lamp0Pos : Position = {-0.5f,2.0f,1.25f};

float4 LightColor = {1.0f, 1.0f, 1.0f, 1.0f};

float4 AmbiColor : Ambient = {0.17f, 0.17f, 0.17f, 1.0f};

float4 SurfColor1 : Diffuse = {1.0f, 0.4f, 0.0f, 1.0f};

float4 SurfColor2 : Diffuse = {0.0f, 0.2f, 1.0f, 1.0f};

float Ks = 0.5;

float SpecExpon : SpecularPower = 25.0;

float SWidth = 10.0;

float Scale = 10.0;

float Balance = 0.5;

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
	float3 Position	: POSITION;
	float4 UV		: TEXCOORD0;
	float4 Normal	: NORMAL;
	float4 Tangent	: TANGENT0;
	float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
	float4 HPosition	: POSITION;
	float4 TexCoord	: TEXCOORD0;
	float3 LightVec	: TEXCOORD1;
	float3 WorldNormal	: TEXCOORD2;
	float3 WorldEyeVec	: TEXCOORD3;
	float3 WorldTangent	: TEXCOORD4;
	float3 WorldBinorm	: TEXCOORD5;
	float4 ObjPos	: TEXCOORD6;
};

/*********** vertex shader ******/

vertexOutput mainVS(appdata IN) {
	vertexOutput OUT;
	OUT.WorldNormal = mul(IN.Normal,matWorldInv).xyz;
	OUT.WorldTangent = mul(IN.Tangent,matWorldInv).xyz;
	OUT.WorldBinorm = mul(IN.Binormal,matWorldInv).xyz;
	float4 Po = float4(IN.Position.xyz,1.0);
	float3 Pw = mul(Po,matWorld).xyz;
	OUT.LightVec = Lamp0Pos - Pw;
	OUT.TexCoord = IN.UV;
	OUT.WorldEyeVec = normalize(matViewInv[3].xyz - Pw);
	float4 hpos = mul(Po,matWorldViewProj);
	OUT.ObjPos = Po;
	OUT.HPosition = hpos;
	return OUT;
}

/********* pixel shader ********/

// PS with box-filtered step function
float4 checkerPS(vertexOutput IN) : COLOR {
	float3 Ln = normalize(IN.LightVec);
	float3 Nn = normalize(IN.WorldNormal);
	float edge = Scale*Balance;
	float op = SWidth/Scale;

	// x stripes
	float width = abs(ddx(IN.ObjPos.x)) + abs(ddy(IN.ObjPos.x));
	float w = width*op;
	float x0 = IN.ObjPos.x/Scale - (w/2.0);
	float x1 = x0 + w;
	float nedge = edge/Scale;
	float i0 = (1.0-nedge)*floor(x0) + max(0.0, frac(x0)-nedge);
	float i1 = (1.0-nedge)*floor(x1) + max(0.0, frac(x1)-nedge);
	float check = (i1 - i0)/w;
	check = min(1.0,max(0.0,check));

	// y stripes
	width = abs(ddx(IN.ObjPos.y)) + abs(ddy(IN.ObjPos.y));
	w = width*op;
	x0 = IN.ObjPos.y/Scale - (w/2.0);
	x1 = x0 + w;
	nedge = edge/Scale;
	i0 = (1.0-nedge)*floor(x0) + max(0.0, frac(x0)-nedge);
	i1 = (1.0-nedge)*floor(x1) + max(0.0, frac(x1)-nedge);
	float s = (i1 - i0)/w;
	check = abs(check - min(1.0,max(0.0,s)));

	// z stripes
	width = abs(ddx(IN.ObjPos.z)) + abs(ddy(IN.ObjPos.z));
	w = width*op;
	x0 = IN.ObjPos.z/Scale - (w/2.0);
	x1 = x0 + w;
	nedge = edge/Scale;
	i0 = (1.0-nedge)*floor(x0) + max(0.0, frac(x0)-nedge);
	i1 = (1.0-nedge)*floor(x1) + max(0.0, frac(x1)-nedge);
	s = (i1 - i0)/w;
	check = abs(check - min(1.0,max(0.0,s)));

	float4 dColor = lerp(SurfColor1,SurfColor2,check);
	float3 Vn = normalize(IN.WorldEyeVec);
	float3 Hn = normalize(Vn + Ln);
	float4 lighting = lit(dot(Ln,Nn),dot(Hn,Nn),SpecExpon);
	float hdn = lighting.z; // Specular coefficient
	float ldn = lighting.y; // Diffuse coefficient
	float4 diffContrib = dColor * (ldn*LightColor + AmbiColor);
	float4 specContrib = hdn * LightColor;
	float4 result = diffContrib + (Ks * specContrib);
	return result;
}

/*************/

technique Main
{
	pass p0 
	{		
		VertexShader = compile vs_2_0 mainVS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		PixelShader = compile ps_2_a checkerPS();
	}
}

/***************************** eof ***/
