#include <transform>
#include <fog>
#include <pos>
#include <normal>
#include <tangent>
#define INDOOR
#ifdef INDOOR
	#include <light>
#else
	float4 vecSunDir;
	float4 vecSunColor;
#endif
float4 vecLight;	

texture entSkin1;	// texture
texture entSkin2;	// normal map
		
const float facAmbient = 0.5; // factors to play with
const float facBump = 1.0;
const float facSpecular = 2.0;

sampler sBaseTex = sampler_state { Texture = <entSkin1>; };
sampler sBump = sampler_state { Texture = <entSkin2>; };

struct specularOut
{
	float4 Pos:    POSITION;
	float  Fog:		FOG;
	float4 Color:	COLOR0;
	float4 Ambient: COLOR1;
	float2 Tex:   	TEXCOORD0;
	float2 Bump:   TEXCOORD1;
	float3 Light:	TEXCOORD2;
	float3 Halfway:TEXCOORD3;
};

specularOut specularVS(
in float4 inPos: POSITION, 
in float3 inNormal: NORMAL,
in float2 inTex: TEXCOORD0,
in float3 inTangent: TEXCOORD2)
{
	specularOut Out;
	Out.Pos	= DoTransform(inPos);
	Out.Tex	= inTex;
	Out.Bump = inTex;	// different coordinate sets required for ps_1_1
	Out.Ambient = facAmbient*vecLight;
	Out.Fog	= DoFog(inPos);
	
	float3 P = DoPos(inPos);

	float3 lightVec;
#ifdef INDOOR
	lightVec = normalize(vecLightPos[0].xyz - P);
	Out.Color = facSpecular*vecLightColor[0];
#else
	lightVec = -vecSunDir;
	Out.Color = facSpecular*vecSunColor;
#endif
			
	CreateTangents(inNormal,inTangent);
		
	// transform the output values into the 0..1 range
	Out.Light = facBump*DoTangent(lightVec) * 0.5 + 0.5;

	float3 EyeDir = normalize(vecViewPos - P) + lightVec;
	Out.Halfway = DoTangent(EyeDir) * 0.5 + 0.5;

	return Out;		
}


float4 specularPS(specularOut In): COLOR
{
	float4 base = tex2D(sBaseTex,In.Tex);
	float3 bumpNormal = tex2D(sBump,In.Bump);
	
	float light = saturate(dot(In.Halfway*2 - 1,bumpNormal*2 - 1)); 
	light *= light;
	light *= light;
	light *= light;
	light *= light;
	light += saturate(dot(In.Light*2 - 1,bumpNormal*2 - 1));
	return base * (In.Ambient + light*In.Color);
}

technique specular
{
	pass one
	{		
		VertexShader = compile vs_1_1 specularVS();
		PixelShader = compile ps_1_1 specularPS();
	}
}

technique fallback { pass one { } }
